/*
=================================
HEADER
=================================
*/

this.name					= "hiredGuns_system";
this.author					= "Thargoid";
this.copyright				= "Creative Commons: attribution, non-commercial, sharealike.";
this.description			= "Set up the player escorts when bought";
this.version				= "1.1";


/*
===========================
Start of wrapper for JSHINT 
============================
*/

(
	function(){


/*
===========================
DIRECTIVE
===========================
*/

"use strict";


/*
===========================
GLOBALS
===========================
*/

this.debug = false;				// There is a 'this.debug' also in hiredGuns_system.js.

if ( this.debug === true ) {
	this.logging = true;
}



/*
===========================
FUNCTIONS
===========================
*/

/*
--------------
Debugging
--------------
*/

this.$dbg = function(msg)
{
	log(this.name,"DEBUG: " + msg);
};


this.startUp = function()
{
	missionVariables.hiredGuns_chance = Math.random() + (system.government * 0.05);
	// That value above is set also in this.shipExitedWitchspace.
	if ( this.debug ) {
		this.$dbg("Debugging ON");
	}
};

this.shipWillDockWithStation = function(station)
{
	if (!station.isMainStation) {
		return;
	}

	// Have we gone anywhere?
	if (system.ID == missionVariables.hiredGuns_launchSystem) {
		return;
	}

	var escortArray = system.shipsWithRole("hiredGuns_escort");
	if (escortArray.length > 0) {
		for (var i = 0; i < escortArray.length; i++) {
			escortArray[i].remove();
		}
	}
};

this.shipWillEnterWitchspace = function()
{
	var escortArray = system.shipsWithRole("hiredGuns_escort");
	if (escortArray.length > 0) {
		for (var i = 0; i < escortArray.length; i++) {
			escortArray[i].remove();
			missionVariables.hiredGuns_count += 1;
		}
	}
};

this.alertConditionChanged = function()
{
	if (player.alertCondition == 3 && player.alertHostiles && missionVariables.hiredGuns_purchased) {
		var escortArray = system.shipsWithRole("hiredGuns_escort", player.ship, 35000);
		if (escortArray.length > 0)
		{
			for (var i = 0; i < escortArray.length; i++) { 
				escortArray[i].AIState = ("FIND_PLAYER_HOSTILES");
			}
		}
	}
};

this.shipExitedWitchspace = function()
{
	if (missionVariables.hiredGuns_count > 0) {
		this.escorts = system.addShips(missionVariables.hiredGuns_role, missionVariables.hiredGuns_count, player.ship.position, 20000);
		for (var i = 0; i < missionVariables.hiredGuns_count; i++) {
			this.escorts[i].scannerDisplayColor1 = "yellowColor";
			this.escorts[i].scannerDisplayColor2 = "magentaColor";
		}
	}
	missionVariables.hiredGuns_chance = Math.random() + (system.government * 0.05);
	missionVariables.hiredGuns_launchSystem = null; // in case we jump out and back to the same system.
	if ( this.debug && system.isInterstellarSpace ) {
		this.$dbg("In interstellar space.");
	}
};

this.playerEnteredNewGalaxy = function(galaxyNumber)
{
	var escortArray = system.shipsWithRole("hiredGuns_escort");
	if (escortArray.length > 0) {
		for (var i = 0; i < escortArray.length; i++) {
			escortArray[i].remove();
		}
	}
	missionVariables.hiredGuns_count = 0;
	missionVariables.hiredGuns_role = null;
	missionVariables.hiredGuns_purchased = null;
	missionVariables.hiredGuns_launchSystem = null;
};

this.shipWillLaunchFromStation = function(station)
{
	if (station.isMainStation && system.countShipsWithRole("hiredGuns_escort") == 0 && missionVariables.hiredGuns_count > 0 && missionVariables.hiredGuns_role) {
		this.escorts = system.addShips(missionVariables.hiredGuns_role, missionVariables.hiredGuns_count, player.ship.position, 20000);
		for (var i = 0; i < missionVariables.hiredGuns_count; i++) { 
			this.escorts[i].scannerDisplayColor1 = "yellowColor";
			this.escorts[i].scannerDisplayColor2 = "magentaColor";
			this.escorts[i].AIState = "INITIALISE_FROM_STATION";
		}
		this.removeEq();
	}
};

this.shipEnteredStationAegis = function(station)
{
	if (!this.ship || !this.ship.isValid) { return; }
	this.ship.addCollisionException(station);
	 // Only main stations have aegis, but just to be safe:
	if (!station.isMainStation) {
		return;
	}

	// Have we gone anywhere?
	if (system.ID == missionVariables.hiredGuns_launchSystem) {
		return;
	}

	if ( this.debug ) {
		this.$dbg("Mission accomplished - removing escorts.");
	}

	var escortArray = system.shipsWithRole("hiredGuns_escort");
	if (escortArray.length > 0) {
		for (var i = 0; i < escortArray.length; i++) { 
			escortArray[i].AIState = ("MISSION_ACCOMPLISHED");
		}
	}
	missionVariables.hiredGuns_count = 0;
	missionVariables.hiredGuns_role = null;
	missionVariables.hiredGuns_purchased = null;
};

this.playerBoughtEquipment = function(equipment)
{
	switch(equipment) {
		case "EQ_HIREDGUN_LOW":
		{
			missionVariables.hiredGuns_role = "hiredGuns_escortLow";
			this.admin();
			player.ship.removeEquipment("EQ_HIREDGUN_LOW");
			break;
		}
		case "EQ_HIREDGUN_MID":
		{
			missionVariables.hiredGuns_role = "hiredGuns_escortMid";
			this.admin();
			player.ship.removeEquipment("EQ_HIREDGUN_MID");
			break;
		}
		case "EQ_HIREDGUN_HIGH":
		{
			missionVariables.hiredGuns_role = "hiredGuns_escortHigh";
			this.admin();
			player.ship.removeEquipment("EQ_HIREDGUN_HIGH");
			// break;
		}
	}
};

this.guiScreenChanged = function(newScreen, oldScreen)
{
	if (oldScreen == "GUI_SCREEN_EQUIP_SHIP") {
		this.removeEq();
	}
};

this.removeEq = function()
{
	if (player.ship.equipmentStatus("EQ_HIREDGUN_LOW") != "EQUIPMENT_UNAVAILABLE") {
		player.ship.removeEquipment("EQ_HIREDGUN_LOW");
		return;
	}
	if (player.ship.equipmentStatus("EQ_HIREDGUN_MID") != "EQUIPMENT_UNAVAILABLE") {
		player.ship.removeEquipment("EQ_HIREDGUN_MID");
		return;
	}
	if (player.ship.equipmentStatus("EQ_HIREDGUN_HIGH") != "EQUIPMENT_UNAVAILABLE") {
		player.ship.removeEquipment("EQ_HIREDGUN_HIGH");
		return;
	}
};

this.admin = function()
{
	missionVariables.hiredGuns_count = 2;
	missionVariables.hiredGuns_launchSystem = system.ID;
	missionVariables.hiredGuns_purchased = true;
};


/*
=========================
End of wrapper for JSHINT
=========================
*/

}).call(this);


// EOF